"""
Copyright (c) 2015, 2020 Tim Waugh <tim@cyberelk.net>

## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.

## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.

## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
"""

from journal_brief.journal_brief import SelectiveReader, LatestJournalEntries
from journal_brief.filter import JournalFilter
from journal_brief.format import list_formatters, get_formatter
from journal_brief.config import Config


__all__ = ['SelectiveReader', 'LatestJournalEntries',
           'JournalFilter',
           'list_formatters', 'get_formatter',
           'Config']

__version__ = '1.1.8'  # also update setup.py and python-journal-brief.spec
